@@ -1,6 +1,8 @@
 from __future__ import division, print_function
 import numpy
+import scipy as sp
 from scipy.stats import nanmean, nanmedian
+from copy import copy
 import sys
 import logging
 
@@ -29,7 +31,7 @@ def sortrows(a, i):
     return b
 
 
-def nan(r, c):
+def nans(r, c):
     a = numpy.empty((r, c))
     a[:] = numpy.nan
     return a
@@ -49,7 +51,7 @@ def max_with_indices(d):
     return ml, mi
 
 
-def icoshift(xt,  xp,  inter='whole',  n='f',  options=[1,  1,  0,  0,  0], scale=None, coshift_preprocessing=False,
+def icoshift(xt,  xp,  inter='whole',  n='f', scale=None, coshift_preprocessing=False,
              coshift_preprocessing_max_shift=None, fill_with_previous=True, average2_multiplier=3):
     '''
     interval Correlation Optimized shifting
@@ -98,21 +100,25 @@ def icoshift(xt,  xp,  inter='whole',  n='f',  options=[1,  1,  0,  0,  0], scal
                          1 : using previous point (default)
                      (2) turns on Co-shift preprocessing
                          0 : no Co-shift preprocessing (default)
-                         1 : Executes a Co-shift step before carrying out iCOshift
-                     (3) max allowed shift for the Co-shift preprocessing (default = equal to n if not specified)
+                         1 :
+                     (3)
                          it has to be given in scale units if option(5)=1
                      (4) 0 : intervals are given in No. of datapoints  (deafult)
                          1 : intervals are given in ppm --> use scale for inter and n
     scale           : vector of scalars used as axis for plot (optional)
+    coshift_preprocessing (bool) (optional; default=False): Execute a Co-shift step before carrying out iCOshift
+    coshift_preprocessing_max_shift (int) (optional): Max allowed shift for the Co-shift preprocessing
+                                                      (default = equal to n if not specified)
+    fill_with_previous (bool) (optional; default=True): Fill using previous point (default); set to False to np.nan
+    average2_multiplier (int) (optional; default=3): Multiplier used for the average2 algorithm
 
-    average2_multiplier: int (optional)
-                    If 'average2' is used as the target, this determines the multiplier used
     OUTPUT
     xcs  (np * mt): shift corrected vector or matrix
     ints (ni * 4) : defined intervals (Int. No.,  starting point,  ending point,  size)
     ind  (np * ni): matrix of indexes reporting how many points each spectrum
                     has been shifted for each interval (+ left,  - right)
     target (1 x mp): actual target used for the final alignment
+
     Authors:
     Francesco Savorani - Department of Food Science
                          Quality & Technology - Spectroscopy and Chemometrics group
@@ -124,12 +130,13 @@ def icoshift(xt,  xp,  inter='whole',  n='f',  options=[1,  1,  0,  0,  0], scal
                          Faculty of Life Sciences
                          University of Copenhagen - Denmark
     email: giorgio.tomasi@ec.europa.eu - www.igm.life.ku.dk
-    Python conversion by:
+
+    Python implementation by:
     Martin Fitzpatrick -  Rheumatology Research Group
                           Centre for Translational Inflammation Research
                           School of Immunity and Infection
                           University of Birmingham - United Kingdom
-
+    email: martin.fitzpatrick@gmail.com
 
     170508 (FrSa) first working code
     211008 (FrSa) improvements and bugs correction
@@ -153,6 +160,10 @@ def icoshift(xt,  xp,  inter='whole',  n='f',  options=[1,  1,  0,  0,  0], scal
 
     # RETURNS [xcs, ints, ind, target]
 
+    # Take a copy of the xp vector since we mangle it somewhat
+    xp = copy(xp)
+
+
     if scale is None:
         using_custom_scale = False
         scale = numpy.array(range(0, xp.shape[1]))
@@ -167,15 +178,13 @@ def icoshift(xt,  xp,  inter='whole',  n='f',  options=[1,  1,  0,  0,  0], scal
         flag_di_scale = numpy.any(abs(dec_scale) > 2 * numpy.min(abs(dec_scale)))
 
         if len(scale) != max(scale.shape):
-            logging.error('scale must be a vector')
+            raise(Exception, 'scale must be a vector')
 
         if max(scale.shape) != xp.shape[1]:
-            logging.error('x and scale are not of compatible length %d vs. %d' %
-                         (max(scale.shape), xp.shape[1]))
+            raise(Exception, 'x and scale are not of compatible length %d vs. %d' % (max(scale.shape), xp.shape[1]))
 
         if inc_scale == 0 or not numpy.all(numpy.sign(dec_scale) == - numpy.sign(inc_scale)):
-            logging.error('scale must be strictly monotonic')
-
+            raise(Exception, 'scale must be strictly monotonic')
 
     if coshift_preprocessing_max_shift is None:
         coshift_preprocessing_max_shift = n
@@ -193,31 +202,31 @@ def icoshift(xt,  xp,  inter='whole',  n='f',  options=[1,  1,  0,  0,  0], scal
     xt_basis = xt
 
     if xt == 'average':
-        xt = numpy.array([nanmean(xp, axis=0), ])
+        xt = nanmean(xp, axis=0).reshape(1, -1)
 
     elif xt == 'median':
-            xt = numpy.array([nanmedian(xp, axis=0), ])
+            xt = nanmedian(xp, axis=0).reshape(1, -1)
 
     elif xt == 'average2':
-            xt = numpy.array([nanmean(xp, axis=0), ])
+            xt = nanmean(xp, axis=0).reshape(1,-1)
             avg2_flag = True
 
     elif xt == 'max':
-            xt = numpy.zeros((1, xp.shape[1],))
+            xt = numpy.zeros((1, xp.shape[1]))
             max_flag = True
 
     nt, mt = xt.shape
     np, mp = xp.shape
 
     if mt != mp:
-        logging.error('Target "xt" and sample "xp" must have the same number of columns')
+        raise(Exception, 'Target "xt" and sample "xp" must have the same number of columns')
 
     if is_number(inter):
         if inter > mp:
-            logging.error('Number of intervals "inter" must be smaller than number of variables in xp')
+            raise(Exception, 'Number of intervals "inter" must be smaller than number of variables in xp')
 
     # Set defaults if the settings are not set
-    options = [options[oi] if oi < len(options) else d for oi, d in enumerate([1, 1, 0, 0, 0]) ]
+    # options = [options[oi] if oi < len(options) else d for oi, d in enumerate([1, 1, 0, 0, 0]) ]
 
     if using_custom_scale:
         prec = abs(numpy.min(numpy.unique(dec_scale)))
@@ -234,17 +243,23 @@ def icoshift(xt,  xp,  inter='whole',  n='f',  options=[1,  1,  0,  0,  0], scal
         if max_flag:
             xt = nanmean(xp, axis=0)
 
-        xp, nil, wint, _ = icoshift(xt, xp, 'whole', coshift_preprocessing_max_shift, [0, 1, 0], scale=scale,
-                                    fill_with_previous=True, average2_multiplier=average2_multiplier )
+        co_shift_scale = scale if using_custom_scale else None
+        xp, nil, wint, _ = icoshift(xt, xp, 'whole',
+                                    coshift_preprocessing=False,
+                                    coshift_preprocessing_max_shift=coshift_preprocessing_max_shift,
+                                    scale=co_shift_scale,
+                                    fill_with_previous=True,
+                                    average2_multiplier=average2_multiplier )
 
-        if xt_basis == 'average':
-            xt = nanmean(xp)
+        if xt_basis == 'average' or xt_basis == 'average2':
+            xt = nanmean(xp).reshape(1, -1)
 
         elif xt_basis == 'median':
-            xt = nanmedian(xp)
+            xt = nanmedian(xp).reshape(1, -1)
+
+        else:  # max?
+            xt = xt.reshape(1,-1)
 
-        elif xt_basis == 'average2':
-            xt = nanmean(xp)
 
     whole = False
     flag2 = False
@@ -260,7 +275,7 @@ def icoshift(xt,  xp,  inter='whole',  n='f',  options=[1,  1,  0,  0,  0], scal
             interv = sort(scal2pts(float(cat(0, interv[:])), scale, prec))
 
             if interv.size != 2:
-                logging.error('Invalid range for reference signal')
+                raise(Exception, 'Invalid range for reference signal')
 
             inter = range(interv[0], (interv[1] + 1))
             flag2 = True
@@ -282,18 +297,28 @@ def icoshift(xt,  xp,  inter='whole',  n='f',  options=[1,  1,  0,  0,  0], scal
         # e.g. 5 intervals on 32768 to match MATLAB algorithm should be
         #0, 6554, 6554, 13108, 13108, 19662, 19662, 26215, 26215, 32767
 
-        step = int(round(float(mp) / inter))
-        segments = []
+        # Distributes vars_left_over in the first "vars_left_over" intervals
+        # startint     = [(1:(N+1):(remain - 1)*(N+1)+1)'; ((remain - 1) * (N + 1) + 1 + 1 + N:N:mP)'];
 
-        for o in range(0, mp, step):
+        remain = mp % inter
+        step = int( float(mp) / inter )
+        segments = []
+        o = 0
+
+        while o < mp:
             segments.extend([o, o])
+            if remain > 0:
+                o += 1
+                remain -=1
+            o += step
 
         # Chop of duplicate zero
         segments = segments[1:]
         segments.append(mp)  # Add on final step
-
         inter = numpy.array(segments, dtype=int).reshape(1, -1)
 
+        logging.info("Calculated intervals: %s" % inter)
+
     elif isinstance(inter, list):  # if is a list of tuples ; add else
         inter = np.array(inter)
 
@@ -308,18 +333,18 @@ def icoshift(xt,  xp,  inter='whole',  n='f',  options=[1,  1,  0,  0,  0], scal
                 inter = inter[:].T
 
     else:
-        logging.error('The number of intervals must be "whole", an integer, or a list of tuples of integers')
+        raise(Exception, 'The number of intervals must be "whole", an integer, or a list of tuples of integers')
 
 
     nint, mint = inter.shape
     scfl = numpy.array_equal(numpy.fix(scale), scale) and not using_custom_scale
 
     if isinstance(inter, basestring) and n not in ['b', 'f']:
-        logging.error('"n" must be a scalar b or f')
+        raise(Exception, '"n" must be a scalar b or f')
 
     elif isinstance(n, int) or isinstance(n, float):
         if n <= 0:
-            logging.error('Shift(s) "n" must be larger than zero')
+            raise(Exception, 'Shift(s) "n" must be larger than zero')
 
         if scfl and not isinstance(n, int):
             logging.warn('"n" must be an integer if scale is ignored; first element (i.e. %d) used' % n)
@@ -329,9 +354,9 @@ def icoshift(xt,  xp,  inter='whole',  n='f',  options=[1,  1,  0,  0,  0], scal
                 n = dscal2dpts(n, scale, prec)
 
         if not flag2 and numpy.any(numpy.diff(numpy.reshape(inter, (2, mint // 2)), 1, 0) < n):
-            logging.error('Shift "n" must be not larger than the size of the smallest interval')
+            raise(Exception, 'Shift "n" must be not larger than the size of the smallest interval')
 
-    flag = numpy.isnan(cat(0, xt, xp))
+    flag = numpy.isnan(cat(0, xt.reshape(1, -1), xp))
     frag = False
     ref = lambda e: numpy.reshape(e, (2, max(e.shape) / 2)).T
     vec = lambda a: a.flatten()
@@ -352,7 +377,7 @@ def icoshift(xt,  xp,  inter='whole',  n='f',  options=[1,  1,  0,  0,  0], scal
             intern_ = remove_nan(
                 numpy.array([0, pma - pmi]).reshape(1, -1), cat(0, xt[:, inter], xp[:, inter]), select)
             if intern_.shape[0] != 1:
-                logging.error('Reference region contains a pattern of missing that cannot be handled consistently')
+                raise(Exception, 'Reference region contains a pattern of missing that cannot be handled consistently')
 
             elif not numpy.array_equal(intern_, numpy.array([1, inter[-2] - inter[0] + 1]).reshape(1, -1)):
                 logging.warn('The missing values at the boundaries of the reference region will be ignored')
@@ -364,13 +389,11 @@ def icoshift(xt,  xp,  inter='whole',  n='f',  options=[1,  1,  0,  0,  0], scal
             intern_ = vec(intern_.T).T
 
         if 0 in intern_.shape:
-            logging.error('Cannot handle this pattern of missing values.')
+            raise(Exception, 'Cannot handle this pattern of missing values.')
 
         if max(intern_.shape) != max(inter.shape) and not flag2:
             if whole:
                 if max(intern_.shape) > 2:
-                    if options[0] == 2:
-                        xt_bu = xt
 
                     xseg, in_or = extract_segments(cat(0, xt, xp), ref(intern_))
                     InOrf = in_or.flatten()
@@ -403,19 +426,19 @@ def icoshift(xt,  xp,  inter='whole',  n='f',  options=[1,  1,  0,  0,  0], scal
             amax, bmax = max_with_indices( numpy.sum(xp) )
             xt[mi:ma] = xp[bmax, mi:ma]
 
-        ind = numpy.nan(np, 1)
+        ind = nans(np, 1)
         missind = not all(numpy.isnan(xp), 2)
-        xcs[missind, :], ind[missind], _ = coshifta(xt, xp[missind,:], inter, n, options,
-                                                    fill_with_previous=fill_with_previous, block_size=block_size)
+        xcs[missind, :], ind[missind], _ = coshifta(xt, xp[missind,:], inter, n, fill_with_previous=fill_with_previous,
+                                                    block_size=block_size)
         ints = numpy.array([1, mi, ma]).reshape(1, -1)
 
     else:
         if mint > 1:
             if mint % 2:
-                logging.error('Wrong definition of intervals ("inter")')
+                raise(Exception, 'Wrong definition of intervals ("inter")')
 
             if ma > mp:
-                logging.error('Intervals ("inter") exceed samples matrix dimension')
+                raise(Exception, 'Intervals ("inter") exceed samples matrix dimension')
 
             # allint=[(1:round(mint/2))' inter(1:2:mint)' inter(2:2:mint)'];
             # allint =
@@ -461,33 +484,26 @@ def icoshift(xt,  xp,  inter='whole',  n='f',  options=[1,  1,  0,  0,  0], scal
                 logging.info('Co-shifting interval no. %s of %s...' % (i, allint.shape[0]) )
 
             # FIXME? 0:2, or 1:2?
-            intervalnow = xp[:, allint[i, 1]:allint[i, 2] + 1]
+            intervalnow = xp[:, allint[i, 1]:allint[i, 2]]
 
             if max_flag:
-                amax, bmax = max_with_indices( numpy.sum(intervalnow, axis=1) )
+                amax, bmax = max_with_indices(numpy.sum(intervalnow, axis=1))
                 target = intervalnow[bmax, :]
-                print('...')
-                print(i)
-                print(xt.shape)
-                print(target.shape)
-                print(allint[i, 1])
-                print(allint[i, 2])
-                print('...')
-                xt[0, allint[i, 1]:allint[i, 2] + 1] = target
+                xt[0, allint[i, 1]:allint[i, 2]] = target
             else:
-                target = xt[:, allint[i, 1]:allint[i, 2] + 1]
+                target = xt[:, allint[i, 1]:allint[i, 2]]
 
             missind = ~numpy.all(numpy.isnan(intervalnow), axis=1)
 
             if not numpy.all(numpy.isnan(target)) and numpy.any(missind):
 
-                cosh_interval, loc_ind, _ = coshifta(target, intervalnow[missind, :], 0, n, options,
+                cosh_interval, loc_ind, _ = coshifta(target, intervalnow[missind, :], 0, n,
                                                      fill_with_previous=fill_with_previous, block_size=block_size)
-                xcs[missind, allint[i, 1]:allint[i, 2] + 1] = cosh_interval
+                xcs[missind, allint[i, 1]:allint[i, 2]] = cosh_interval
                 ind[missind, i] = loc_ind.flatten()
 
             else:
-                xcs[:, allint[i, 1]:allint[i, 1] + 1] = intervalnow
+                xcs[:, allint[i, 1]:allint[i, 1]] = intervalnow
 
         if avg2_flag:
 
@@ -498,16 +514,17 @@ def icoshift(xt,  xp,  inter='whole',  n='f',  options=[1,  1,  0,  0,  0], scal
                     logging.info('Co-shifting again interval no. %d of %d... ' % (i, allint.shape[0]))
 
                 intervalnow = xp[:, allint[i, 1]:allint[i, 2]]
-                target1 = numpy.mean(xcs[:, allint[i, 1]:allint[i, 2]+1], axis=0)
+                target1 = numpy.mean(xcs[:, allint[i, 1]:allint[i, 2]], axis=0)
                 min_interv = numpy.min(target1)
                 target = (target1 - min_interv) * average2_multiplier
                 missind = ~numpy.all(numpy.isnan(intervalnow), 1)
 
                 if (not numpy.all(numpy.isnan(target))) and (numpy.sum(missind) != 0):
-                    cosh_interval, loc_ind, _ = coshifta(target, intervalnow[missind, :], 0, n, options,
+                    cosh_interval, loc_ind, _ = coshifta(target, intervalnow[missind, :], 0, n,
                                                          fill_with_previous=fill_with_previous, block_size=block_size)
                     xcs[missind, allint[i, 1]:allint[i, 2]] = cosh_interval
-                    xt[allint[i, 1]:allint[i, 2]] = target
+
+                    xt[0, allint[i, 1]:allint[i, 2]] = target
                     ind[missind, i] = loc_ind.T
 
                 else:
@@ -515,7 +532,7 @@ def icoshift(xt,  xp,  inter='whole',  n='f',  options=[1,  1,  0,  0,  0], scal
 
     if frag:
 
-        xn = numpy.nan(np, mp)
+        xn = nans(np, mp)
         for i_sam in range(0, np):
             for i_seg in range(0, in_or.shape[0]):
                 xn[i_sam, in_or[i_seg, 0]:in_or[i_seg, 1]
@@ -529,19 +546,17 @@ def icoshift(xt,  xp,  inter='whole',  n='f',  options=[1,  1,  0,  0,  0], scal
                         if flag_nan[i_seg, 1, i_sam]:
                             xn[i_sam, (in_or[i_seg, 1] - loc_ind[i_sam, 0] + 1):in_or[i_seg, 1]+1] = numpy.nan
 
-        if options[0] == 2:
-            xt = xt_bu
 
         xcs = xn
     target = xt
 
     if flag_coshift:
-        ind = ind + wint * ones(1, ind.shape[1])
+        ind = ind + wint * numpy.ones( (1, ind.shape[1]) )
 
     return xcs, ints, ind, target
 
 
-def coshifta(xt, xp, ref_w=0, n=numpy.array([1, 2, 3]), options=[], fill_with_previous=True, block_size=(2 ** 25)):
+def coshifta(xt, xp, ref_w=0, n=numpy.array([1, 2, 3]), fill_with_previous=True, block_size=(2 ** 25)):
 
     if ref_w == 0 or ref_w.shape[0] == 0:
         ref_w = numpy.array([0])
@@ -553,7 +568,7 @@ def coshifta(xt, xp, ref_w=0, n=numpy.array([1, 2, 3]), options=[], fill_with_pr
         rw = 1
 
     if fill_with_previous:
-        filling = - numpy.inf
+        filling = -numpy.inf
 
     else:
         filling = numpy.nan
@@ -575,20 +590,19 @@ def coshifta(xt, xp, ref_w=0, n=numpy.array([1, 2, 3]), options=[], fill_with_pr
     logging.info('mt=%d, mp=%d' % (mt, mp))
 
     if mt != mp:
-        logging.error('Target "xt" and sample "xp" must be of compatible size (%d, %d)' % (mt, mp) )
+        raise(Exception, 'Target "xt" and sample "xp" must be of compatible size (%d, %d)' % (mt, mp) )
 
     if numpy.any(n <= 0):
-        logging.error('shift(s) "n" must be larger than zero')
+        raise(Exception, 'shift(s) "n" must be larger than zero')
 
     if nr != 1:
-        logging.error('Reference windows "ref_w" must be either a single vector or 0')
+        raise(Exception, 'Reference windows "ref_w" must be either a single vector or 0')
 
     if rw > 1 and (numpy.min(ref_w) < 1) or (numpy.max(ref_w) > mt):
-        logging.error('Reference window "ref_w" must be a subset of xp')
+        raise(Exception, 'Reference window "ref_w" must be a subset of xp')
 
     if nt != 1:
-        logging.error('Target "xt" must be a single row spectrum/chromatogram')
-
+        raise(Exception, 'Target "xt" must be a single row spectrum/chromatogram')
 
     auto = 0
     if n == 'b':
@@ -615,9 +629,9 @@ def coshifta(xt, xp, ref_w=0, n=numpy.array([1, 2, 3]), options=[], fill_with_pr
         try_last = 1
 
     if nt != 1:
-        logging.error('ERROR: Target "xt" must be a single row spectrum/chromatogram')
+        raise(Exception, 'ERROR: Target "xt" must be a single row spectrum/chromatogram')
 
-    xw = nan(np, mp)
+    xw = nans(np, mp)
     ind = numpy.zeros((1, np))
 
     n_blocks = int(numpy.ceil(sys.getsizeof(xp) / block_size))
@@ -637,35 +651,35 @@ def coshifta(xt, xp, ref_w=0, n=numpy.array([1, 2, 3]), options=[], fill_with_pr
 
             elif numpy.isnan(filling):
                 # FIXME
-                xtemp = numpy.array(
-                    [nan(np, n), xp, nan(np, n)]).reshape(1, -1)
+                xtemp = cat(1,
+                    nans(np, n), xp, nans(np, n))
 
             if rw == 1:
-                ref_w = range(0, mp)
+                ref_w = numpy.arange(0, mp) #.reshape(1,-1)
 
-            ind = nan(np, 1)
+            ind = nans(np, 1)
             r = False
 
+
             for i_block in range(0, n_blocks):
                 block_indices = range(
                     ind_blocks[i_block], ind_blocks[i_block + 1])
 
-                _, ind[block_indices], ri = cc_fft_shift(xt[0, [ref_w]], xp[block_indices, :][:, ref_w],
+                _, ind[block_indices], ri = cc_fft_shift(xt[0, ref_w].reshape(1,-1), xp[block_indices, :][:, ref_w],
                                                          numpy.array([-n, n, 2, 1, filling]) )
 
                 if not r:
                     r = numpy.empty((0, ri.shape[1]))
                 r = cat(0, r, ri).T
 
-            temp_index = range(-n, n)
+            temp_index = range(-n, n+1)
 
             for i_sam in range(0, np):
                 index = numpy.flatnonzero(temp_index == ind[i_sam])
-
                 xw[i_sam, :] = xtemp[i_sam, index:index + mp]
 
             if (numpy.max(abs(ind)) == n) and try_last != 1:
-                if n + src_step >= ref_w.shape[1]:
+                if n + src_step >= ref_w.shape[0]:
                     try_last = 1
                     continue
                 n += src_step
@@ -682,27 +696,23 @@ def coshifta(xt, xp, ref_w=0, n=numpy.array([1, 2, 3]), options=[], fill_with_pr
 
     else:
         if filling == -numpy.inf:
-
-            xtemp = numpy.array([
-                                numpy.repeat(xp[:, 0], n),
-                                xp,
-                                numpy.repeat(xp[:, mp-1], n),
-                                ]).reshape(1, -1)
+            xtemp = cat(1, numpy.tile(xp[:, :1], (1., n)),
+                        xp, numpy.tile(xp[:, -1:, ], (1., n)))
 
         elif numpy.isnan(filling):
-            xtemp = numpy.array([nan(np, n), xp, nan(np, n)]).reshape(1, -1)
+            xtemp = cat(1,
+                nans(np, n), xp, nans(np, n))
+
 
         if rw == 1:
-            ref_w = range(0, mp)
+            ref_w = numpy.arange(0, mp) #.reshape(1,-1)
 
-        ind = nan(np, 1)
+        ind = nans(np, 1)
         r = numpy.array([])
-
         for i_block in range(n_blocks):
 
-
             block_indices = range(ind_blocks[i_block], ind_blocks[i_block + 1])
-            dummy, ind[block_indices], ri = cc_fft_shift(xt[0, [ref_w]], xp[block_indices, :][:, ref_w],
+            dummy, ind[block_indices], ri = cc_fft_shift(xt[0, ref_w].reshape(1,-1), xp[block_indices, :][:, ref_w],
                                                          numpy.array([-n, n, 2, 1, filling]))
             r = cat(0, r, ri)
 
@@ -718,8 +728,7 @@ def coshifta(xt, xp, ref_w=0, n=numpy.array([1, 2, 3]), options=[], fill_with_pr
     return xw, ind, r
 
 
-
-def defints(xp, interv, opt):
+def defints(xp, interv):
     np, mp = xp.shape
     sizechk = mp / interv - round(mp / interv)
     plus = (mp / interv - round(mp / interv)) * interv
@@ -730,7 +739,7 @@ def defints(xp, interv, opt):
     else:
         logging.warn('Size of the last interval = %d' % (interv + plus))
 
-    if opt[0] != 0 and (sizechk != 0):
+    if sizechk != 0:
         logging.info('The last interval will not fulfill the selected intervals size "inter"=%f.' % interv)
         logging.info('Size of the last interval = %f ' % plus)
 
@@ -743,7 +752,6 @@ def defints(xp, interv, opt):
     return inter
 
 
-
 def cc_fft_shift(t, x=False, options=numpy.array([])):
 
     dim_x = numpy.array(x.shape)
@@ -753,15 +761,13 @@ def cc_fft_shift(t, x=False, options=numpy.array([])):
     options = numpy.array([options[oi] if oi < len(options) else d for oi, d in enumerate(options_default)])
     options[numpy.isnan(options)] = options_default[numpy.isnan(options)]
 
-    logging.info(options)
-
     if options[0] > options[1]:
-        logging.error('Lower bound for shift is larger than upper bound')
+        raise(Exception, 'Lower bound for shift is larger than upper bound')
 
     time_dim = int(options[2] - 1)
 
     if dim_x[time_dim] != dim_t[time_dim]:
-        logging.error('Target and signals do not have compatible dimensions')
+        raise(Exception, 'Target and signals do not have compatible dimensions')
 
     ord_ = numpy.array(
         [time_dim] +
@@ -773,15 +779,11 @@ def cc_fft_shift(t, x=False, options=numpy.array([])):
     x_fft = numpy.transpose(x, ord_)  # permute
     x_fft = numpy.reshape(x_fft, (dim_x[time_dim], numpy.prod(dim_x[ord_[1:]])))
 
-    b = numpy.array(
-        [
-            range(1, (numpy.prod(dim_x[ord_[1:]]) + 1)),
-            range(1, (numpy.prod(dim_x[ord_[1:]]) + 1)),
-            numpy.sqrt(numpy.nansum(x_fft ** 2, axis=0))
-        ]).T
-
     # FIXME? Sparse/dense switchg
-    x_fft = x_fft / b[:, -1]
+    p = numpy.arange(0, numpy.prod(dim_x[ ord_[1:] ] ) )
+    s = numpy.max(p) + 1
+    b = sp.sparse.dia_matrix( (1.0/numpy.sqrt(numpy.nansum(x_fft ** 2, axis=0)), [0]), shape=(s,s) ).todense()
+    x_fft = numpy.dot(x_fft, b)
 
     t = numpy.transpose(t, ord_)
     t = numpy.reshape(t, (dim_t[time_dim], numpy.prod(dim_t[ord_[1:]])))
@@ -789,29 +791,28 @@ def cc_fft_shift(t, x=False, options=numpy.array([])):
 
     np, mp = x_fft.shape
     nt = t.shape[0]
-
-    flag_miss = numpy.any(numpy.isnan(x_fft[:])) or numpy.any(numpy.isnan(t[:]))
+    flag_miss = numpy.any(numpy.isnan(x_fft)) or numpy.any(numpy.isnan(t))
 
     if flag_miss:
+
         if len(x.shape) > 2:
-            logging.error('Multidimensional handling of missing not implemented, yet')
-        miss_off = nan(1, mp)
+            raise(Exception, 'Multidimensional handling of missing not implemented, yet')
+        miss_off = nans(1, mp)
 
         for i_signal in range(0, mp):
 
             limits = remove_nan(
-                numpy.array([0, np - 1]).reshape(1, -1), x_fft[:, i_signal].T, numpy.all)
+                numpy.array([0, np - 1]).reshape(1, -1), x_fft[:, i_signal].reshape(1, -1), numpy.all)
 
-            if not numpy.array_equal(limits.shape, numpy.array([1, 2]).reshape(1, -1)):
-                logging.error(
-                    'Missing values can be handled only if leading or trailing')
+            if limits.shape != (2, 1):
+                raise(Exception, 'Missing values can be handled only if leading or trailing')
 
             if numpy.any(cat(1, limits[0], mp - limits[1]) > numpy.max(abs(options[0:2]))):
-                logging.error('Missing values band larger than largest admitted shift')
+                raise(Exception, 'Missing values band larger than largest admitted shift')
 
             miss_off[i_signal] = limits[0]
 
-            if miss_off[i_signal] > 1:
+            if numpy.any(miss_off[i_signal-1] > 1):
                 x_fft[0:limits[1] - limits[0] + 1,
                       i_signal] = x_fft[limits[0]:limits[1], i_signal]
 
@@ -873,7 +874,7 @@ def cc_fft_shift(t, x=False, options=numpy.array([])):
         if options[3] == 1:
             cc = numpy.squeeze(numpy.prod(cc, axis=1))
         else:
-            logging.error('Invalid options for correlation of multivariate signals')
+            raise(Exception, 'Invalid options for correlation of multivariate signals')
 
     pos = cc.argmax(axis=0)
     values = cat(1, numpy.reshape(shift, (len(shift), 1)), cc)
@@ -882,7 +883,7 @@ def cc_fft_shift(t, x=False, options=numpy.array([])):
     if flag_miss:
         shift = shift + miss_off
 
-    x_warp = nan(*[dim_x[0]] + list(dim_t[1:]))
+    x_warp = nans(*[dim_x[0]] + list(dim_t[1:]))
     ind = numpy.tile(numpy.nan, (len(x.shape), 18))
     indw = ind
 
@@ -962,7 +963,7 @@ def remove_nan(b, signal, select=numpy.any, flags=False):
     2.00.00 23 Mar 09 -> Added output for adjacent nan's in signals
     2.01.00 23 Mar 09 -> Added select input parameter
     '''
-    c = nan(b.shape[0], b.shape[1] if len(b.shape) > 1 else 1)
+    c = nans(b.shape[0], b.shape[1] if len(b.shape) > 1 else 1)
     b = b.reshape(1, -1)
     count = 0
     signal = numpy.isnan(signal)
@@ -977,22 +978,22 @@ def remove_nan(b, signal, select=numpy.any, flags=False):
             a = numpy.flatnonzero(p < 0) + 1
             b = numpy.flatnonzero(p > 0)
 
-            if not in_[0]:
+            if numpy.any(~in_[0]):
                 a = cat(1, numpy.array([0]), a)
 
             else:
                 b = b[1:]
 
-            if not in_[-1]:
+            if numpy.any(~in_[-1]):
                 b = cat(1, b, numpy.array([max(ind.shape) - 1]))
 
             a = numpy.unique(a)
             b = numpy.unique(b)
 
-            d = ind[cat(0, a[:].reshape(1, -1), b[:].reshape(1, -1))]
+            d = ind[cat(0, a, b)]
 
             c.resize(d.shape)
-            c[count:count + max(a.shape), :] = d
+            c[count:count + max(a.shape) + 1] = d
 
             count = count + max(a.shape)
 
@@ -1052,10 +1053,10 @@ def normalise(x, flag=False):
         flag = numpy.any(~p_att[:])
 
     else:
-        p_att = not numpy.isnan(x)
+        p_att = ~numpy.isnan(x)
 
     m, n = x.shape
-    xn = nan(m, n)
+    xn = nans(m, n)
     if flag:
         for i_n in range(0, n):
             n = numpy.linalg.norm(x[p_att[:, i_n], i_n])
@@ -1117,21 +1118,21 @@ def extract_segments(x, segments):
     flag_ab = numpy.any(Sd < 2)
 
     if flag_si:
-        logging.error('Segment boundary matrix must have two columns')
+        raise(Exception, 'Segment boundary matrix must have two columns')
 
     if flag_in:
-        logging.error('Segment boundaries must be integers')
+        raise(Exception, 'Segment boundaries must be integers')
 
     if flag_ob:
-        logging.error('Segment boundaries outside of segment')
+        raise(Exception, 'Segment boundaries outside of segment')
 
     if flag_ni:
-        logging.error('segments boundaries must be monotonically increasing')
+        raise(Exception, 'segments boundaries must be monotonically increasing')
 
     if flag_ab:
-        logging.error('segments must be at least two points long')
+        raise(Exception, 'segments must be at least two points long')
 
-    xseg = nan(n, q)
+    xseg = nans(n, q)
     origin = 0
     segnew = []
 
@@ -1148,9 +1149,13 @@ def extract_segments(x, segments):
     return xseg, segnew
 
 
+def find_nearest(array, value):
+    idx = (numpy.abs(array-value)).argmin()
+    return array[idx], idx
+
 def scal2pts(ppmi,  ppm=[],  prec=None):
     """
-    Transforms scalars in data points
+    Transforms scalars into data points
 
     pts = scal2pts(values, scal)
 
@@ -1177,70 +1182,31 @@ def scal2pts(ppmi,  ppm=[],  prec=None):
     1.00.00 12 Feb 09 -> First working version
     1.01.00 11 Mar 09 -> Added input parameter check
     """
+    rev = ppm[0] > ppm[1]
+
     if prec is None:
         prec = min(abs(unique(numpy.diff(ppm))))
 
-    dimppmi = ppmi.shape
-    ppmi = ppmi[:]
-    ppm = ppm[:]
-    rev = ppm[0] > ppm[1]
+    pts = []
+    for i in ppmi:
+        nearest_v, idx = find_nearest(ppm, i)
+        if abs(nearest_v-i) > prec:
+            pts.append(numpy.nan)
+        else:
+            pts.append( idx )
 
-    if rev:
-        ppm = ppm[-1:0:1]
-
-    ubound = (ppmi - ppm[-1]) < prec & (ppmi - ppm[-2]) > 0
-    lbound = (ppm[0] - ppmi) < prec & (ppm[0] - ppmi) > 0
-    ppmi[(ubound - 1)] = ppm[-1]
-    ppmi[(lbound - 1)] = ppm[0]
-
-    if nargin < 2:
-        logging.error('Not enough input arguments')
-
-    if max(ppmi.shape) > max(ppm.shape):
-        logging.warn('scal2pts ppm vector is shorter than the values')
-
-    xxi, k = sort(ppmi[:])
-    nil, j = sort(numpy.array([ppm[:], xxi[:]]).reshape(1, -1))
-    r[j] = range(1, (max(j.shape) + 1))
-    r = r[(max(ppm.shape) + 1):] - range(1, max(ppmi.shape) + 1)
-    r[k] = r
-    r[ppmi == ppm[-1]] = max(ppm.shape)
-    ind = numpy.flatnonzero((r > 0) & (r <= max(ppm.shape)))
-    ind = ind[:]
-    pts = Inf(ppmi.shape)
-    pts[ind] = r[ind]
-
-    ptsp1 = numpy.min(max(ppm.shape), abs(pts + 1))
-    ptsm1 = numpy.max(1, abs(pts - 1))
-    ind = numpy.flatnonzero(isfinite(pts))
-
-    dp0 = abs(ppm[pts[ind]] - ppmi[ind])
-    dpp1 = abs(ppm[ptsp1[ind]] - ppmi[ind])
-    dpm1 = abs(ppm[ptsm1[ind]] - ppmi[ind])
-
-    pts[ind[dpp1 < dp0]] += 1
-    pts[ind[dpm1 < dp0]] += 1
-
-    if 0 in pts.shape:
-        pts = numpy.array([])
-    pts[~numpy.isfinite(pts)] = nan
-
-    if rev:
-        pts = max(ppm.shape) - pts + 1
-    if not numpy.array_equal(pts.shape, ppmi):
-        pts = numpy.reshape(pts, dimppmi)
-
-    return pts
+    return numpy.array(pts)
 
 
-def dscal2dpts(d, ppm, **args):
+
+def dscal2dpts(d, ppm, prec=None):
     """
     Translates an interval width from scal to the best approximation in sampling points.
 
     i = dppm2dpts(delta, scal, prec)
 
     INPUT
-    delta: interval widths in scale units
+    delta: interval width in scale units
     scal : scale
     prec : precision on the scal axes
 
@@ -1257,19 +1223,18 @@ def dscal2dpts(d, ppm, **args):
 
     Last modified: 28th October,  2013
     """
-    if 0 in d.shape:
-        i = numpy.array([])
-        return i
+    if d == 0:
+        return 0
 
     if d <= 0:
-        logging.error('delta must be positive')
+        raise(Exception, 'delta must be positive')
 
-    if ppm[0] < ppm[1]:
-        i = scal2pts(ppm[0] + d, ppm, varargin[:]) - 1
+    if ppm[0] < ppm[1]: # Scale in order
+        i = scal2pts(numpy.array([ppm[0] + d]), ppm, prec) -1
 
     else:
-        i = max(ppm.shape) - scal2pts(ppm[-2] + d, ppm, args[:]) + 1
+        i = max(ppm.shape) - scal2pts(numpy.array([ppm[-1] + d]), ppm, prec) +1
 
-    return i
+    return i[0]
 
 
